'****************************************************************
'*  Name    : Test_LED_4620_HSM.BAS                             *
'*  Author  : [Alberto Freixanet]                               *
'*  Notice  : Copyright (c) 2016 [AFreixanet]                   *
'*          : All Rights Reserved                               *
'*  Date    : 19/11/2017                                        *
'*  Version : 5.0 (with internal oscillator)                    *
'*  Notes   : Blinking a LED                                    *
'*          : with any board.                                   *
'*  Code powered by the Proton Development Suite Version 3.6.0.3*
'*          : This special code is used to check the bootloader *
'*          : Using the Internal Oscillator: A delay of 25mS    *
'*          : is mandatory after setting the OSCCON SFR         *
'****************************************************************

'===============================================================================
' Header file for use with the Amicus18 board
' Same device as the Bootloader.
    Device = 18F4620
    Declare Xtal = 40 

    ' DEFINE THE PLL: On / OFF
    $define PLL_ConfigFuses On

    ' Enable the to use the internal Oscillator.
    ' Uncomment the next line to enable the configuration.
    '$define _InternalOSC_ 

    ' Enable the Line to use the CLOCK OUT FUNCTION: 
    ' If CLKOUT function is enabled, CLKOUT on RA6 & Port function On RA7 otherwize Port function On RA6 & RA7.     
    '$Define _CLKOUT_Function_    

'===============================================================================
' COPY the Declares of the bootloader:
    Declare Optimiser_Level = 2
    Declare Dead_Code_Remove = On
    Declare Watchdog = On
    Declare Bootloader = OFF
'===============================================================================
' Define the PORT & PIN of the test LED.
'
    $define _PORT PORTC
    $define _PIN 5
'===============================================================================
' The Declare UART Baudrate is automatic depending of the Xtal to be syncronized with the bootloader.
    $ifndef _BaudRate
        $if _xtal >= 40
            $ifdef _InternalOSC_
            $define _BaudRate 57600
            $else
            $define _BaudRate 115200
            $endif
        $endif
        $if (_xtal >= 16) And (_xtal < 40)
            $define _BaudRate 57600
        $endif
        $if _xtal < 16
            $define _BaudRate 38400
        $endif
    $endif
'===============================================================================
' Same Baudrate as the Bootloader to check the Software.
'-------------------------------------------------------------------------------
$if _BaudRate = 9600
    Declare Hserial_Baud = 9600
    Declare Hserial_TXSTA = 36      ' Enable transmit and asynchronous mode
    Declare Hserial_RCSTA = 144     ' Enable serial port and continuous receive
    Declare Hserial_Clear = On      ' Enable Error clearing on received characters
$endif
'-------------------------------------------------------------------------------
$if _BaudRate = 19200
    Declare Hserial_Baud = 19200
    Declare Hserial_TXSTA = 36      ' Enable transmit and asynchronous mode
    Declare Hserial_RCSTA = 144     ' Enable serial port and continuous receive
    Declare Hserial_Clear = On      ' Enable Error clearing on received characters
$endif
'-------------------------------------------------------------------------------
$if _BaudRate = 38400
    Declare Hserial_Baud = 38400
    Declare Hserial_TXSTA = 36      ' Enable transmit and asynchronous mode
    Declare Hserial_RCSTA = 144     ' Enable serial port and continuous receive
    Declare Hserial_Clear = On      ' Enable Error clearing on received characters
$endif
'-------------------------------------------------------------------------------
$if _BaudRate = 57600
    Declare Hserial_Baud = 57600
    Declare Hserial_TXSTA = 36      ' Enable transmit and asynchronous mode  4024
    Declare Hserial_RCSTA = 144     ' Enable serial port and continuous receive
    Declare Hserial_Clear = On      ' Enable Error clearing on received characters
$endif
'-------------------------------------------------------------------------------
$if _BaudRate = 115200
    Declare Hserial_Baud = 115200
    Declare Hserial_TXSTA = 36      ' Enable transmit and asynchronous mode
    Declare Hserial_RCSTA = 144     ' Enable serial port and continuous receive
    Declare Hserial_Clear = On      ' Enable Error clearing on received characters
$endif
'===============================================================================
' Copy the Config Fuses from the Bootloader:
'-------------------------------------------------------------------------------
Config_Start
$if PLL_ConfigFuses = On
    $ifdef _InternalOSC_
        $if (_xtal = 16) Or (_xtal = 32)
            $ifdef _CLKOUT_Function_
            OSC = INTIO7    ;Internal Oscillator with FOSC/4 Output on RA6 and  port function I/O on RA7
            $else
            OSC = INTIO67	;Internal oscillator block, port function on RA6 and RA7
            $endif
        $else
            $error "Error in (Declare Xtal) with internal oscillator & PLL_ConfigFuses = On"
        $endif
    $else
        $if (_xtal >= 10) And (_xtal <= 40)
        OSC = HSPLL	;HS oscillator, PLL enabled (Clock Frequency = 4 x FOSC1)(Xtal piece 2,5-16 MHz)
        $else
            $error "Error in (Declare Xtal) with external Xtal & PLL_ConfigFuses = On"
        $endif
    $endif
$endif
$if PLL_ConfigFuses = OFF
    $ifdef _InternalOSC_
        $error "Error in (Declare Xtal) with internal oscillator & PLL_ConfigFuses = OFF"
    $else
        $if (_xtal >= 10) And (_xtal <= 25)
        OSC = HS	    ;HS oscillator PLL disabled  (medium power 4-20 MHz) (Xtal piece 10-25 MHz)
        $else
            $if (_xtal > 25) And (_xtal <= 40)
                $ifdef _CLKOUT_Function_
                OSC = EC    ; External EC oscillator; Clock with FOSC/4 Output on RA6
                $else
                OSC = ECIO6  ;External Clock with I/O function on RA6
                $endif
            $else
                $error "Error in (Declare Xtal) with external Xtal & PLL_ConfigFuses = OFF"
            $endif
       $endif
    $endif
$endif
    FCMEN = Off	;Fail-Safe Clock Monitor disabled
    IESO = Off	;Oscillator Switchover mode disabled
    PWRT = On	;PWRT enabled
    BOREN = On	;Brown-out Reset enabled and controlled by software (SBOREN is enabled)
    BORV = 3	; Minimum setting
    WDT = OFF	      ;WDT disabled (control is placed on the SWDTEN bit)
    WDTPS = 1024	;1:1024
    CCP2MX = PORTC	;CCP2 input/output is multiplexed with RC1
    PBADEN = On	    ;PORTB<4:0> pins are configured as analog I/O on Reset
    LPT1OSC = OFF	;Timer1 configured for higher power operation
    MCLRE = On	;MCLR pin enabled; RE3 input pin disabled
    STVREN = On	;Stack full/underflow will cause Reset
    LVP = OFF	;Single-Supply ICSP disabled
    XINST = OFF	;Instruction set extension and Indexed Addressing mode disabled (Legacy mode)
    Debug = OFF	;Background debugger disabled, RB6 and RB7 configured as general purpose I/O pins
    Cp0 = OFF	;Block 0 (000800-003FFFh) not code-protected
    CP1 = OFF	;Block 1 (004000-007FFFh) not code-protected
    CP2 = OFF	;Block 2 (008000-00BFFFh) not code-protected
    CP3 = OFF	;Block 3 (00C000-00FFFFh) not code-protected
    CPB = OFF	;Boot block (000000-0007FFh) not code-protected
    CPD = On	;Data EEPROM code-protected
    WRT0 = OFF	;Block 0 (000800-003FFFh) not write-protected
    WRT1 = OFF	;Block 1 (004000-007FFFh) not write-protected
    WRT2 = OFF	;Block 2 (008000-00BFFFh) not write-protected
    WRT3 = OFF	;Block 3 (00C000-00FFFFh) not write-protected
    WRTC = On	;Configuration registers (300000-3000FFh) write-protected
    WRTB = OFF	;Boot Block (000000-0007FFh) not write-protected
    WRTD = OFF	;Data EEPROM not write-protected
    EBTR0 = OFF	;Block 0 (000800-003FFFh) not protected from table reads executed in other blocks
    EBTR1 = OFF	;Block 1 (004000-007FFFh) not protected from table reads executed in other blocks
    EBTR2 = OFF	;Block 2 (008000-00BFFFh) not protected from table reads executed in other blocks
    EBTR3 = OFF	;Block 3 (00C000-00FFFFh) not protected from table reads executed in other blocks
    EBTRB = OFF	;Boot Block (000000-0007FFh) not protected from table reads executed in other blocks
Config_End
'###############################################################################
' THE USER SOFTWARE STARTS:
'###############################################################################
'===============================================================================
' New Definitions of the library flags.
    Dim F_TMR2IF As PIR1Bits_TMR2IF     ' TMR2 To PR2 Match Interrupt Flag Bit
    Dim F_TMR2IE As PIE1Bits_TMR2IE     ' TMR2 To PR2 Match Interrupt Enable bi
    Dim F_TMR2ON As T2CONBits_TMR2ON    ' Timer2 On Bit 
    Dim F_IPEN As RCONBits_IPEN         ' Disable priority levels On interrupts (PIC16CXXX Compatibility mode)
    Dim F_TMR2IP As IPR1Bits_TMR2IP     ' TMR2 to PR2 Match Interrupt Priority bit
'------------------------------------------------------------------------------- 
'ASCII characters  
    Symbol Home = 1
    Symbol Bell = 7
    Symbol BkSp = 8
    Symbol Tab = 9
    Symbol CR = 13
    Symbol LF = $0A     ' Line Feed Char    
    Symbol True = 1     ' Boolean True
    Symbol False = 0    ' Boolean False
    Symbol Null = 0     ' Null character 
    Symbol ACK = 6
    Symbol STX = 2
    Symbol ETX = 3
    Symbol SPACE = $20
    Symbol DEL = 127    
    Symbol ESC = $1B    ' Escape
    Symbol BS = 08      ' Backspace 
'===============================================================================
' Declare some variables.
    Dim LoopByte As Byte
    Dim Temp1 As Byte
    Dim Temp2 As Byte
    Dim DataWord As Word
    Dim EepAddress As Word
    Dim PICPartNumber As Word
    Dim PICRevision As Byte
    Dim PostScaler As Byte
    Dim CounterEep As Byte
    Dim NoASCII As Byte
    Dim IDLOCS_String As String * 8    
'===============================================================================
' Running with Internal oscillator ONLY.
' CONFIG FUSES:
' FOSC = INTIO67    ;Internal oscillator block, port function on RA6 and RA7
' HFOFST = OFF	    ;HFINTOSC output and ready status are delayed by the oscillator stable status
' Time for HF-INTOSC to Stabilize: 1mS, Time For PLL to Stabilize: 2mS
' Delay for code: 25mS (more or less)
'*******************************************************************************
'* SAME CONFIGURATION OF THE INTOASC AS THE BOOTLOADER                         *
'*******************************************************************************
$ifdef _InternalOSC_
    ' See the Datasheet to configure the internal oscillator.
    '===========================================================================
INT_OSC:
    ' OSCCON: OSCILLATOR CONTROL REGISTER
    'bit 6-4 IRCF<2:0>: Internal Oscillator Frequency Select bits
    '    111 = 8 MHz (INTOSC drives clock directly)
    '    110 = 4 MHz
    '    101 = 2 MHz
    '    100 = 1 MHz(3)
    '    011 = 500 kHz
    '    010 = 250 kHz
    '    001 = 125 kHz
    '    000 = 31 kHz (from either INTOSC/256 or INTRC directly)(2)
    '
    $if PLL_ConfigFuses = On
        $if _xtal = 32
        OSCCON = %01110000      ' 8Mhz x 4. Internal oscillator: Sleep, 32Mhz, R, R, Primary clock. 
        OSCTUNEBits_PLLEN = 1   ' PLL enabled (FOSC = 4 x INTOSC)
        $endif
        $if _xtal = 16
        OSCCON = %01100000      ' 4Mhz x 4. Internal oscillator: Sleep, 16Mhz, R, R, Primary clock. 
        OSCTUNEBits_PLLEN = 1   ' PLL enabled (FOSC = 4 x INTOSC)
        $endif
    $endif

    $if PLL_ConfigFuses = OFF
        $error "NO internal oscillator for this Declare Xtal & PLL_ConfigFuses = OFF"
    $endif
    '---------------------------------------------------------------------------
    OSCTUNEBits_INTSRC = 1              ' 1 = 31.25 kHz device clock derived from 8 MHz HFINTOSC source (divide-by-512 enabled) (if necessary)
    '---------------------------------------------------------------------------
    ' Waiting until the internal Oscillator is stable.
    $ifdef OSCCONBits_IOFS
    While OSCCONBits_IOFS = 0
        #ifdef WatchDog_Req
        Clrwdt
        #endif    
    Wend    ' Wait untill OSC is stable (not a good job)
    $else
        $ifdef OSCCONBits_HFIOFS
        While OSCCONBits_HFIOFS = 0
            #ifdef WatchDog_Req
            Clrwdt
            #endif         
        Wend  ' Wait untill OSC is stable (not a good job)
        $endif
    $endif
    DelayMS 25                          ' Delay to stabilize the Internal Oscillator. (Mandatory)
    '===========================================================================
$endif

$ifndef _InternalOSC_
    ' FOSC WITH EXTERNAL CRISTAL
    '---------------------------------------------------------------------------
    ' Not sure. Need to be verified.
    OSCTUNEBits_INTSRC = 1   ' 1 = 31.25 kHz device clock derived from 8 MHz HFINTOSC source (divide-by-512 enabled) (if necessary)
    '---------------------------------------------------------------------------
    DelayMS 12                          ' Delay to stabilize the Xtal Oscillator. (Mandatory)
$endif
'===============================================================================
    Include "D:\MyIncFiles\Devicelist.inc"
'===============================================================================
    Include "D:\MyIncFiles\HRsout2StopBits.inc"
'===============================================================================
    $define ERASE_FLASH_BLOCKSIZE 64        ' For the PIC18F26K22.
'*******************************************************************************
'* Name    : On_Low_Interrupt                                                  *
'* Purpose : To test the ISR_Low Jump by the Bootloader.                       *
'*         : Clock = FOSC/4 => 64Mhz/4 = 16Mhz                                 *
'*******************************************************************************
    On_Low_Interrupt GoTo ISR_Low
    GoTo OverLowInterrupt
'===============================================================================   
ISR_Low:
    ' Timer is used to Toggle the LED9 every second.
    Context Save
    If INTCONBits_TMR0IF = 1 Then   ' Check the Timer0 Overflow flag
        INTCONBits_TMR0IF = 0       ' Clear the Timer0 Overflow flag
        #ifdef WatchDog_Req
        Clrwdt
        #endif
        Toggle _PORT._PIN                 ' LED = One second ON, one second OFF.
    EndIf
    Context Restore
'===============================================================================
OverLowInterrupt:     
'===============================================================================
MAIN_TEST:
    #ifdef WatchDog_Req
    Clrwdt
    #endif
    TBLPTRU = 0
    DelayMS 25  ' Delay for the internal Oscillator to stabilize (important for Baudrate).
'-------------------------------------------------------------------------------    
    INTCON2 = %10000000     ' PORTB Pullups disabled
    $ifdef _WPUB
    WPUB = 0
    $endif
    INTCON3 = 0
    PIR2 = 0
    PIE2 = 0
    IPR2 = 0                     
'
' Declare all digital   
    ADCON0 = 0
    ADCON1 = 15
    CMCON = 7
    ADCON2 = 0    
    CMCON = 0
    SSPCON1 = 0
    $ifdef _WUP
    CCP1CON = 0
    $endif
    $ifdef _CM1CON
    CCP1CON = 0
    $endif
    $ifdef _CM2CON
    CCP2CON = 0
    $endif
    $ifdef _CM1CON0
    CM1CON0 = 0
    $endif
    $ifdef _CM2CON0
    CM2CON0 = 0
    $endif
    $ifdef _CM2CON1    
    CM2CON1 = 0
    $endif
    $ifdef _ANSELA
    ANSELA = 0
    $endif
    $ifdef _ANSELB
    ANSELB = 0
    $endif
    $ifdef _ANSELC
    ANSELC = 0
    $endif
    $ifdef _ANSELD
    ANSELD = 0
    $endif    
'
' The eeprom is not used for the moment.
    EECON1 = 0
'
TRISPORT:
' PORTS
    ' TRIS the LED pin
    Output _PORT._PIN
    ' Clear all PORTS
    LATA = 0
    LATB = 0
    LATC = 0
  
'-------------------------------------------------------------------------------
    DelayMS 50  ' Time to stabilize the Internal oscillator.
'*******************************************************************************
' Write some String in the Data eeprom memory.
E2p_AddressString1 EData "Testing the PIC18 AGV Bootloader V5.0 written in High Side of the ROM.",0

E2p_AddressString2 EData "New Universal PIC18 Bootloader by Alberto Freixanet. ",0

E2p_AddressString3 EData "A 8 digits Password could be needed to download the user code. ",0
 
'*******************************************************************************
STARTING_TEST:
'===============================================================================
' Configuration of the Timer0 to toggle a LED every 1.048Sec 
    ' Timer0 is configured As a 16-Bit timer/Counter, 1:256 prescale value: 1.048Sec 
    ' For xtal = 64
    INTCON2.7 = 1               ' All PORTB pull-ups are disabled.
    RCONBits_IPEN = 1           ' Enable priority levels on interrupts.
    IPR1Bits_TMR2IP = 1         ' Timer2 (Debounce Library) for High Interrupt.
    $if _xtal = 64
    T0CON = %00000111           ' /256 Timer0 is set for 1 second (more or less) with FOSC = 64Mhz.
    $endif
    $if (_xtal > 16) And (_xtal <= 48)
    T0CON = %00000110           ' /128 Timer0 is set for 0,84 second (more or less) with FOSC = >16-48Mhz.
    $endif
    $if (_xtal = 16) Or (_xtal = 14)    
    T0CON = %00000101           ' /64 Timer0 is set for 1 second (more or less) with FOSC = 14-16Mhz.
    $endif
    $if (_xtal = 10) Or (_xtal = 12)    
    T0CON = %00000100           ' /32 Timer0 is set for 1 second (more or less) with FOSC = 12-10Mhz.
    $endif
    INTCON2Bits_TMR0IP = 0      ' Timer0 for Low Interrupt
    INTCONBits_TMR0IF = 0       ' Clear the Timer0 interrupt flag to begin.
    INTCONBits_TMR0IE = 1       ' Enable the Timer0 interrupts.
    INTCON.7 = 1                ' Enables all high priority interrupts
    INTCON.6 = 1                ' Enables all low priority interrupts
    TMR0H = 0                   ' Clear the Timer0 counter.
    Nop
    TMR0L = 0
    Nop    
    T0CONBits_TMR0ON = 1        ' Start the Timer0.
    PostScaler = 0              ' Not used for the moment.
'===============================================================================
    HRSOut CR,LF,CR,LF
'-------------------------------------------------------------------------------    
    HRSOut "USER CODE STARTS!",CR,LF 
'===============================================================================
' Read the WDT config in memory map from PPI file.
    EECON1 = %10000000              ' Access flash memory
    Set_Bank TBLPTRL
    Asm
    Movlw Low (Config2H)
    Movwf TBLPTRL
    Movlw High (Config2H)
    Movwf TBLPTRH
    Movlw upper (Config2H)
    Movwf TBLPTRU
    EndAsm
    Tblrd*                          ' Read one memory position.  
    Temp1 = TABLAT                  ' Copy the TABLAT SFR immediatly because it is cleared if read out of this routine??
    EECON1 = 0                      ' Disable Access the Configuration registers
    TBLPTRU = 0                     ' The HRSOut command of the compiler does not clear the "TBLPTRU = 0", then take care of that.
'-------------------------------------------------------------------------------    
    HRSOut "CONFIG FUSES: ",CR,LF 
    $if (_device = _18F23K22) Or (_device = _18F24K22) Or (_device = _18F25K22) Or (_device = _18F26K22) Or (_device = _18F43K22) Or (_device = _18F44K22) Or (_device = _18F45K22) Or (_device = _18F46K22)
    Temp1 = Temp1 & 3
    If Temp1 = 3 Then
        HRSOut "WDT enabled in hardware; SWDTEN bit disabled!"    
    ElseIf Temp1 = 2 Then
        HRSOut "WDT controlled by the SWDTEN bit!"   
    ElseIf Temp1 = 1 Then
        HRSOut "WDT enabled when Device is active, disabled when Device is in Sleep; SWDTEN bit disabled!"
    ElseIf Temp1 = 0 Then
        HRSOut "WDT disabled in hardware; SWDTEN bit disabled!"
    EndIf
    $endif
    $if (_device = _18F242) Or (_device = _18F248) Or (_device = _18F252) Or (_device = _18F258) Or (_device = _18F442) Or (_device = _18F452) Or (_device = _18F458)
    Temp1 = Temp1 & 1
    If Temp1 = 1 Then
        HRSOut "WDT enabled in hardware; SWDTEN bit disabled!"    
    Else
        HRSOut "WDT controlled by the SWDTEN bit!"   
    EndIf
    $endif
    $if (_device = _18F2420) Or (_device = _18F2520) Or (_device = _18F4420) Or (_device = _18F4520)
    Temp1 = Temp1 & 1
    If Temp1 = 1 Then
        HRSOut "WDT enabled in hardware; SWDTEN bit disabled!"    
    Else
        HRSOut "WDT controlled by the SWDTEN bit!"   
    EndIf
    $endif
    $if (_device = _18F2525) Or (_device = _18F2620) Or (_device = _18F4525) Or (_device = _18F4620)
    Temp1 = Temp1 & 1
    If Temp1 = 1 Then
        HRSOut "WDT enabled in hardware; SWDTEN bit disabled!"    
    Else
        HRSOut "WDT controlled by the SWDTEN bit!"   
    EndIf
    $endif
    $if (_device = _18F23K20) Or (_device = _18F24K20) Or (_device = _18F25K20) Or (_device = _18F26K20) Or (_device = _18F43K20) Or (_device = _18F44K20) Or (_device = _18F45K20) Or (_device = _18F46K20)
    Temp1 = Temp1 & 1
    If Temp1 = 1 Then
        HRSOut "WDT enabled in hardware; SWDTEN bit disabled!"    
    Else
        HRSOut "WDT controlled by the SWDTEN bit!"   
    EndIf
    $endif
    HRSOut CR,LF
'-------------------------------------------------------------------------------
' Config Fuses:   BOREN = On    ;Brown-out Reset enabled and controlled by software (SBOREN is enabled)
    If RCONBits_SBOREN = 1 Then
        HRSOut "Brown-out Reset enabled!",CR,LF 
    Else
        HRSOut "Brown-out Reset disabled!",CR,LF     
    EndIf
'------------------------------------------------------------------------------- 
    HRSOut CR,LF
'===============================================================================
' Check the SWDTEN Bit.
    If Temp1 = 3 Or Temp1 = 1 Then
        HRSOut "Be carrefull the WDT is always active!",CR,LF  
    EndIf
    If Temp1 = 2 Then
        HRSOut "After Reset the "
        If WDTCONBits_SWDTEN = 1 Then
            HRSOut "SWDTEN Bit = 1 (error)",CR,LF
        Else
            HRSOut "SWDTEN Bit = 0 (good, by code)",CR,LF
        EndIf
        WDTCONBits_SWDTEN = 1           ' Enable the Watchdog
        If WDTCONBits_SWDTEN = 1 Then
            HRSOut "Now the SWDTEN Bit is set => Watchdog Enabled.",CR,LF
        Else
            HRSOut "Now the SWDTEN Bit is Cleared => Watchdog Disabled.",CR,LF    
        EndIf
    EndIf
'*******************************************************************************
'* Name    : Read the DEVICE ID Locations Address                              *
'* Purpose : Load in RAM the address of the ID LOCATIONS from PPI file.        *
'*           Less configuration for the user.                                  *
'*******************************************************************************
    ' Read the DEVID1 (Device ID) from the PPI file.
    EECON1 = %10000000              ' Access Flash memory 
    Set_Bank TBLPTRL
    Asm
    Movlw Low (DEVID1)
    Movwf TBLPTRL
    Movlw High (DEVID1)
    Movwf TBLPTRH
    Movlw upper (DEVID1)
    Movwf TBLPTRU
    EndAsm
    Tblrd*+                         ' perform table read with post-increment
    DataWord.LowByte = TABLAT
    Tblrd*+                         ' perform table read with post-increment
    DataWord.HighByte = TABLAT        
    TBLPTRU = 0                     ' Cleared for security
    EECON1 = 0                      ' Disable Access the Configuration registers
    '---------------------------------------------------------------------------
    ' Calculate the PIC Part Number & revision
    PICPartNumber = DataWord >> 5
    PICRevision = DataWord.LowByte & 31
'-------------------------------------------------------------------------------
    ' Print the Device Name to the Terminal.
    $ifndef __DEVICE_ID
        $error "The Devicelist.inc is missing!!"
    $endif
    Select PICPartNumber
        Case __DEVICE_ID
            HRSOut "Device = ", __DEVICE_NAME, CR,LF
        Case Else
            HRSOut CR,LF    
            HRSOut "The PIC Device number read from the Board is not correct!",CR,LF   
            HRSOut CR,LF        
    EndSelect   
'-------------------------------------------------------------------------------
    HRSOut "PIC Part Number: ", Dec PICPartNumber, " / $", Hex4 PICPartNumber 
    HRSOut ", Revision: ", Dec PICRevision, " / $", Hex2 PICRevision,CR,LF

    HRSOut "FOSC: ", Dec _xtal," Mhz"
    $ifdef _InternalOSC_
    HRSOut " Internal Oscillator ",CR,LF    
    $else
    HRSOut " Xtal Oscillator ",CR,LF    
    $endif

    $if PLL_ConfigFuses = On
        HRSOut "The PLL is used!",CR,LF
    $else
        $if PLL_ConfigFuses = OFF    
            HRSOut "FOSC without PLL!",CR,LF
        $endif
    $endif

    '---------------------------------------------------------------------------
    ' Read "_CWRITE_BLOCK" from asm file/PPI file.
    Set_Bank Temp1    
    Asm    
    Movlw (_CWRITE_BLOCK)  
    Movwf Temp1
    EndAsm    
    HRSOut "WRITE BLOCK: ", Dec Temp1," Bytes",CR,LF 
    '---------------------------------------------------------------------------
    HRSOut "ERASE BLOCK: ", Dec ERASE_FLASH_BLOCKSIZE," Bytes",CR,LF
    HRSOut "EEPROM MEMORY: ", Dec _eeprom," Bytes",CR,LF
    HRSOut "ROM MEMORY: ", Dec _code," Bytes",CR,LF     
    HRSOut "RAM MEMORY: ", Dec _ram," Bytes",CR,LF
    '---------------------------------------------------------------------------
    ' Read "RAM_BANKS" from asm file/PPI file.
    Set_Bank Temp1    
    Asm    
    Movlw (RAM_BANKS)  
    Movwf Temp1
    EndAsm
    HRSOut "RAM BANKS: ", Dec Temp1,CR,LF     
    '---------------------------------------------------------------------------
    ' Read "BankA_End" from asm file/PPI file.
    Set_Bank Temp1    
    Asm    
    Movlw (BankA_End)  
    Movwf Temp1
    EndAsm
    HRSOut "BANKA END: ", Dec Temp1,CR,LF      
    '---------------------------------------------------------------------------
    HRSOut "Number UART: 1 of ", Dec _uart,CR,LF 
    HRSOut "UART Baud Rate: ", Dec _BaudRate,CR,LF
'-------------------------------------------------------------------------------
    HRSOut CR,LF
'-------------------------------------------------------------------------------    
' READING THE IDLOCS...
    HRSOut "Reading the PIC ID in HEX: "
    ' Read the IDLOC0 (ID Locations) from Asm file/PPI file.
    EECON1 = %10000000              ' Access Flash memory
    Set_Bank TBLPTRL
    Asm
    Movlw Low (IDLOC0)
    Movwf TBLPTRL
    Movlw High (IDLOC0)
    Movwf TBLPTRH
    Movlw upper (IDLOC0)
    Movwf TBLPTRU
    EndAsm
    For LoopByte = 0 To 7           ' Start the LoopByte                    
        Tblrd*+                     ' Read one memory position.
        HRSOut "$"
        HRSOut Hex2 TABLAT          ' Print results in hex format.
        HRSOut " "
    Next  
    EECON1 = 0  ' Disable Access the memory Flash
    TBLPTRU = 0
    HRSOut CR,LF
    '---------------------------------------------------------------------------
    HRSOut "Reading the PIC ID in DEC: "
    ' Repeat loading the IDLOC0 address because the all address values have been
    ' modified by the HRSOut command.
    EECON1 = %10000000              ' Access Flash memory
    Set_Bank TBLPTRL
    Asm
    Movlw Low (IDLOC0)
    Movwf TBLPTRL
    Movlw High (IDLOC0)
    Movwf TBLPTRH
    Movlw upper (IDLOC0)
    Movwf TBLPTRU
    EndAsm
    For LoopByte = 0 To 7           ' Start the LoopByte
        Tblrd*+                     ' Read one memory position.
        HRSOut Dec TABLAT           ' Print results in decimal format.
        HRSOut " "
    Next      
    EECON1 = 0                      ' Disable Access the memory Flash
    TBLPTRU = 0 ' The HRSOut command of the compiler does not clear the "TBLPTRU = 0", then take care of that.
                ' The HRSOut command can crash if the TBLPTRU SFR is not cleared.
    HRSOut CR,LF
'-------------------------------------------------------------------------------
    HRSOut "Reading the PIC ID in ASCII: "
    ' Repeat loading the IDLOC0 address because the all address values have been
    ' modified by the HRSOut command.
    EECON1 = %10000000              ' Access Flash memory
    Set_Bank TBLPTRL
    Asm
    Movlw Low (IDLOC0)
    Movwf TBLPTRL
    Movlw High (IDLOC0)
    Movwf TBLPTRH
    Movlw upper (IDLOC0)
    Movwf TBLPTRU
    EndAsm
    NoASCII = 0
    For LoopByte = 0 To 7           ' Start the LoopByte
        Tblrd*+                     ' Read one memory position.
        If TABLAT > 32 And TABLAT < 127 Then 
            IDLOCS_String[LoopByte] = TABLAT    ' Load results in ASCII format.
        Else
            NoASCII = 1
            Break
        EndIf
    Next      
    EECON1 = 0  ' Disable Access the Configuration registers
    TBLPTRU = 0 ' The HRSOut command of the compiler does not clear the "TBLPTRU = 0", then take care of that.
                ' The HRSOut command can crash if the TBLPTRU SFR is not cleared.
    IDLOCS_String#8 = 0
    If NoASCII = 1 Then
        HRSOut "The ID Locations are not in ASCII Format"
    Else
        HRSOut IDLOCS_String
    EndIf
    HRSOut CR,LF
'-------------------------------------------------------------------------------
STRING_EEP:
    ' Send a String to the terminal read from the data eeprom memory.
    HRSOut CR,LF
    '---------------------------------------------------------------------------
    HRSOut "START EEPROM Strings:",CR,LF
    EepAddress = E2p_AddressString1
    GoSub ReadEEPString_Sub

    EepAddress = E2p_AddressString2
    GoSub ReadEEPString_Sub

    EepAddress = E2p_AddressString3
    GoSub ReadEEPString_Sub
    HRSOut "END EEPROM Strings:",CR,LF,CR,LF  
'-------------------------------------------------------------------------------
    #ifdef __LOW_INTERRUPTS_ENABLED
    HRSOut "Low Interrupts are used!",CR,LF
    #endif
    #ifdef __HIGH_INTERRUPTS_ENABLED
    HRSOut "High Interrupts are used!",CR,LF
    #endif
    HRSOut CR,LF    
'-------------------------------------------------------------------------------
    HRSOut "The LED must toggle now!",CR,LF
    HRSOut CR,LF        
'===============================================================================
MAIN_CODE:
    While 1 = 1
        #ifdef WatchDog_Req
        Clrwdt
        #endif
        DelayMS 10
    Wend
'===============================================================================
'SUBROUTINES:
'===============================================================================
' Read a string written in the eeprom memory.
ReadEEPString_Sub:
    LoopByte = 0
    CounterEep = 0
    While 1 = 1
        #ifdef WatchDog_Req
        Clrwdt
        #endif
        Temp1 = ERead EepAddress + LoopByte
        If Temp1 = 0 Then               ' Check  if final of the string.
            HRSOut CR,LF
            Return 
        EndIf
        If Temp1 = $FF Then             ' Check if this address has been erased.
            Inc CounterEep
            If CounterEep > 20 Then     ' Too much addresses erased.
                HRSOut "This String is erased!",CR,LF
                Return     
            EndIf
        Else
            HRSOut Temp1
        EndIf
        Inc LoopByte
    Wend 
'===============================================================================
    Stop
'===============================================================================
' Write some static data to the ID Locations.
'Asm
'__IDLOCS   IDLOC0,0x88
'__IDLOCS   IDLOC1,0x77
'__IDLOCS   IDLOC2,0x66
'__IDLOCS   IDLOC3,0x55
'__IDLOCS   IDLOC4,0x44
'__IDLOCS   IDLOC5,0x33
'__IDLOCS   IDLOC6,0x22
'__IDLOCS   IDLOC7,0x11
'EndAsm

'Asm
'__IDLOCS   IDLOC0,"L"
'__IDLOCS   IDLOC1,"O"
'__IDLOCS   IDLOC2,"A"
'__IDLOCS   IDLOC3,"D"
'__IDLOCS   IDLOC4,"E"
'__IDLOCS   IDLOC5,"R"
'__IDLOCS   IDLOC6,"5"
'__IDLOCS   IDLOC7,"0"
'EndAsm
'===============================================================================

